<?php

namespace Bolt\Extension\Ginger\Squarepay;

use Bolt\Extension\Ginger\Squarepay\Api;
use Bolt\Extension\Ginger\Squarepay\Twig;
use Bolt\Extension\SimpleExtension;
use Bolt\Menu\MenuEntry;
use Silex\Application;

use Silex\ControllerCollection;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Squarepay extension class.
 *
 * @author Richard Knight <rich@apewave.com>
 */
class SquarepayExtension extends SimpleExtension
{
	private $apiClient = null;
	
	protected function registerServices(Application $app)
	{
        $app['square.apiClient'] = $app->share(function($app) {
			
			// Create and configure a new Square API client using the OAuth token
			$manager = $app['filesystem'];
			if (!$manager->hasFilesystem('extensions_config')) {
				throw new Exception('Could not find filesystem extensions_config');
			}
			$filesystem = $manager->getFilesystem('extensions_config');
			$path = '.sqoatoken';
			$file = $filesystem->getFile($path);
			if (!$file->exists($path)) {
				throw new Exception('Could not find '.$path);
			}
			$oauthToken = $file->read($path);

			$apiConfig = new \SquareConnect\Configuration();
			$apiConfig->setAccessToken($oauthToken);
			$apiClient = new \SquareConnect\ApiClient($apiConfig);
			return new \SquareConnect\ApiClient($apiConfig);
		});

		$app['twig.runtime.square'] = $app->share(function ($app) {
			return new Twig\SquareTwigRuntime(
				$app['square.apiClient'],
				$app['twig'],
				$this->getConfig(),
				$app['session']->get('basket')
			);
		});

		$app['twig.runtimes'] = $app->extend(
			'twig.runtimes',
			function (array $runtimes) {
                // You must append your array to the passed in $runtimes array and return it
                return $runtimes + [
                    Twig\SquareTwigRuntime::class => 'twig.runtime.square',
                ];
			}
		);
	}

	protected function registerFrontendRoutes(ControllerCollection $collection)
	{
        $collection->post ('/basket/add',    [$this, 'basketAdd']);
        $collection->post ('/basket/remove', [$this, 'basketRemove']);
        $collection->post ('/basket/update', [$this, 'basketUpdate']);
        $collection->match('/basket/clear',  [$this, 'basketClear']);
		$collection->match('/process-square', [$this, 'processSquarePaymentResponse']);
	}
	
	protected function registerBackendControllers()
	{
		return [
			'/extensions/square' => new Controller\SquareBackendController($this->getConfig())
		];
	}

	protected function registerMenuEntries()
	{
		$menu = MenuEntry::create('square-menu', 'square')
			->setLabel('Square')
			->setIcon('fa:credit-card')
			->setPermission('settings')
			// ->setRoute('SquarepayExtension')
			;
		return [$menu];
	}

	protected function registerTwigPaths()
    {
		// Register Twig namespace to use in controllers
		return [
            'templates/backend'  => ['namespace' => 'Squareback'],
            'templates/frontend' => ['namespace' => 'Squarefront']
        ];
    }
	
	protected function registerTwigFunctions() {
        $getCatalog = [Twig\SquareTwigRuntime::class, 'getCatalog'];
        $getBasket 	= [Twig\SquareTwigRuntime::class, 'getBasket'];
        $cardform 	= [Twig\SquareTwigRuntime::class, 'displayCCForm'];
		return [
			'getcatalog' 	=> [ $getCatalog ],
			'getbasket'		=> [ $getBasket, ['is_safe' => ['html']] ],
			'cardform' 		=> [ $cardform,  ['is_safe' => ['html']] ]
		];
	}

	// -----------------------------------------------------------------------------------------------------------------
	// BASKET

	public function basketAdd(Application $app, Request $request)
	{
        $basket	= $app['session']->get('basket');
		$sku	= $request->request->get('sku');
		$name	= $request->request->get('name');
		$price	= $request->request->get('price');
		$key	= substr(md5($sku), 0, 10);
		if (isset($basket[$key])) {
			$basket[$key]['quantity']++;
			$basket[$key]['total'] = $basket[$key]['quantity'] * $basket[$key]['price'];
		} else {
			$basket[$key] = [
				'sku'		=> $sku,
				'name'		=> $name,
				'price'		=> $price,
				'quantity'	=> 1,
				'total'		=> $price
			];
		}
        $app['session']->set('basket', $basket);
        return new RedirectResponse('/basket');
	}

	public function basketUpdate(Application $app, Request $request)
	{
		$basket	= $app['session']->get('basket');
		$sku	= $request->request->get('sku');
		$action	= $request->request->get('action');
		$key	= substr(md5($sku), 0, 10);
		if (isset($basket[$key])) {
			if ($action === '+') {
				$basket[$key]['quantity']++;
			} else {
				$basket[$key]['quantity']--;
			}
			$basket[$key]['total'] = $basket[$key]['quantity'] * $basket[$key]['price'];
			if ($basket[$key]['quantity'] === 0) {
				unset($basket[$key]);
			} 
		}
        $app['session']->set('basket', $basket);
        return new RedirectResponse('/basket');
	}

	public function basketRemove(Application $app, Request $request)
	{
        $basket	= $app['session']->get('basket');
		$sku	= $request->request->get('sku');
		$key	= substr(md5($sku), 0, 10);
		unset($basket[$key]);
        $app['session']->set('basket', $basket);
        return new RedirectResponse('/basket');
	}

	public function basketClear(Application $app)
	{
        $app['session']->remove('basket');
        return new RedirectResponse('/basket');
	}

	// -----------------------------------------------------------------------------------------------------------------
	// SQUARE PAYMENT PROCESSING

	public function processSquarePaymentResponse() {
		// TODO
	}

}
